<?php 
/**
 * Controlador informes del rol administrador
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Informes extends Admin_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();
        $this->load->model('incidencia_model');
        $this->load->model('usuari_model');
        $this->load->model('departament_model');
        $this->load->model('pf_model');
    }

    /**
     * Pre-carrega dades i obre vista principal informes 
     */
    public function index()
    {
        // Caixes amb totals
        $this->data['total_incidencies_obertes'] = count($this->incidencia_model->get_incidencies(array('estats'=>array('1','2','3'))));
        $this->data['total_incidencies_tancades'] = count($this->incidencia_model->get_incidencies(array('estats'=>array('4'))));
        $this->data['total_usuaris_actius'] = count($this->usuari_model->get_usuaris(array('actiu'=>'1')));
        $this->data['total_pfs'] = count($this->pf_model->get_pfs());
        
        // Gràfic incidències per departament
        $departaments = $this->departament_model->get_departaments();
        unset($departaments['']);
        foreach($departaments as $departament)
            $this->data['departaments'][] = "'".$departament."'";
        $this->data['departaments'] = join($this->data['departaments'], ',');
        $this->data['departament_incidencies_obertes'] = join($this->_get_incidencies_departaments($departaments, array('1','2','3')), ',');
        $this->data['departament_incidencies_tancades'] = join($this->_get_incidencies_departaments($departaments, array('4')), ',');
        
        // TOP gestors incidències obertes
        $this->data['carrega_gestors'] = $this->incidencia_model->get_carrega_gestors();
        
        
        // Gràfic top motius
        $this->data['motius_incidencies'] = $this->incidencia_model->get_top_motius(30);
        
        // Gràfic activitat temps real
        $this->data['activitat_actual'] = floor($this->incidencia_model->get_activitat_periode(20)->activitat);
        
        // Càrrega vista
        $this->data['extraHead'] = '<script type="text/javascript" src="'.asset_url().'js/highcharts.js"></script>
                                    <script type="text/javascript" src="'.asset_url().'js/highcharts-exporting.js"></script>';
        $this->data['subview'] = 'admin/informes';
        $this->data['page_title'] = 'Informes i estadístiques';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Retorna array anb nº d'incidències segons departament i estat dels últims 30 dies
     * @param {array} $departaments Departaments per filtrar
     * @param {array} $estats Estats per filtrar
     * @return {array} Nº d'incidències ordenades per departament
     */
    private function _get_incidencies_departaments($departaments, $estats)
    {
        foreach($departaments as $id=>$nom)
        {
            $num_incidencies[] = count($this->incidencia_model->get_incidencies(array('departament'=>$id,'estats'=>$estats, 'upd_desde'=>30)));
        }
        return $num_incidencies;
    }

    /**
     * Retorna valor indicant l'activitat recent (incidències i actualitzacions)
     * en els últims 20 segons per actualitzar el gràfic.
     */
    public function get_activitat()
    {
        $total_activitat = $this->incidencia_model->get_activitat_periode(20);
        echo json_encode(floor($total_activitat->activitat));
    }
}

/* Fi del fitxer informes.php */
/* Ubicació: ./application/controllers/admin/informes.php */