<?php 
/**
 * Controlador gestió d'usuaris del rol administrador
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Usuaris extends Admin_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();	
        $this->load->model('usuari_model');
    }

    /**
     * Pàgina principal d'administració d'usuaris
     */
    public function index()
    {
        $filtres = array();
        $filtres['text'] = $this->input->post('text');
        $filtres['rol'] = $this->input->post('rol');
        
        // Prepara la pàginació de la taula d'usuaris
        $this->load->library('pagination');
        $config['base_url'] = base_url().'admin/usuaris';
        $config['per_page'] = '15';
        $config['uri_segment'] = '3';
        $config['total_rows'] = count($this->usuari_model->get_usuaris($filtres));
        $this->pagination->initialize($config);
        $inici = ($this->uri->segment(3)) ? $this->uri->segment(3) : 0;
        $this->data['paginacio'] = $this->pagination->create_links();
        
        $this->data['usuaris'] = $this->usuari_model->get_usuaris($filtres, $config['per_page'], $inici);
        $this->data['usuaris_total'] = $config['total_rows'];
        $this->data['rols'] = $this->usuari_model->get_rols();
        
        $this->data['extraHead'] = '<link href="'.asset_url().'css/bootstrap-rowlink.min.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-rowlink.min.js"></script>';
        $this->data['page_title'] = 'Gestió d\'usuaris';
        $this->data['subview'] = 'admin/usuaris';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Carrega vista Nou Usuari
     * @param {string} $error Error a mostrar a la vista
     */
    public function nou_usuari($error = '')
    {
        $this->load->model('departament_model');
        $this->load->helper('string');
        
        $this->data['error'] = $error;
        $this->data['rols'] = $this->usuari_model->get_rols();
        $this->data['departaments'] = $this->departament_model->get_departaments();
        $this->data['contrasenya'] = strtolower(random_string('alnum', 6));
        $this->data['page_title'] = 'Nou usuari';
        $this->data['subview'] = 'admin/nou_usuari';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Valida i guarda l'usuari enviat
     */
    public function guarda_usuari()
    {   
        $this->load->library('form_validation');
        
        // Validació formulari ok
        if ($this->form_validation->run('admin/usuaris/guarda_usuari') !== FALSE)
        {
            if ($this->input->post('rol')!=='1' && !$this->input->post('departament'))
            {
                $this->nou_usuari('Has d\'especificar un departament');
            }
            else
            {
                // Desar usuari
                $insert = $this->usuari_model->insert_usuari($this->input->post());
                // Insert ok
                if ($insert)
                {
                    // Enviar correu a l'usuari
                    $this->_mail_nou_usuari($this->input->post('nom'), $this->input->post('id_usuari'), $this->input->post('contrasenya'), $this->input->post('email'));

                    $this->messages->add("Usuari creat correctament", 'success', TRUE);
                    redirect('admin/usuaris');
                }
                // Insert error
                else
                {
                    $this->nou_usuari("Error guardant les dades a la base de dades..");
                }
            }
        }
        // Validació formulari error
        else
        {
            $this->nou_usuari();
        }
    }
    
    /**
     * Envia correu electrònic al nou usuari creat amb les dades d'accés
     * @param {string} $nom
     * @param {string} $usuari
     * @param {string} $contrasenya
     * @param {string} $email
     */
    private function _mail_nou_usuari($nom, $usuari, $contrasenya, $email)
    {
        $this->load->library('email');
        
        $this->data['nom'] = $nom;
        $this->data['usuari'] = $usuari;
        $this->data['contrasenya'] = $contrasenya;
        $this->data['link'] = site_url();
        $this->email->from($this->session->userdata('usuari')->email,
                            $this->session->userdata('usuari')->nom.' '.$this->session->userdata('usuari')->cognoms);
        $this->email->to($email);
        $this->email->subject('['.config_item('app_name').']'.' Dades usuari CIAU');
        $this->email->message( $this->load->view('mail/mail_nou_usuari', $this->data, true) );
        $this->email->send();
    }
    
    /**
     * Carrega vista Editar Usuari
     * @param {string} $id_usuari Identificador usuari a editar
     * @param {string} $error Error a mostrar a la vista
     */
    public function editar_usuari($id_usuari, $error = '')
    {
        $this->data['usuari'] = $this->usuari_model->get_usuari($id_usuari);
        
        // Sense ID usuari o usuari no trobat
        if (!$this->data['usuari'])
        {
            $this->messages->add("No s'ha especificat un usuari vàlid", 'error', TRUE);
            redirect('/admin/usuaris');
        }
        
        $this->load->model('departament_model');
        $this->load->helper('string');
        
        $this->data['error'] = $error;
        $this->data['rols'] = $this->usuari_model->get_rols();
        $this->data['departaments'] = $this->departament_model->get_departaments();
        $this->data['extraHead'] = '<script type="text/javascript" src="'.asset_url().'js/bootbox.min.js"></script>
                                    <link href="'.asset_url().'css/bootstrap-datepicker.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-datepicker.js"></script>
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-datepicker-locales/bootstrap-datepicker.ca.js"></script>';
        $this->data['page_title'] = 'Editar usuari '.$id_usuari;
        $this->data['subview'] = 'admin/editar_usuari';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Valida i guarda l'usuari enviat
     */
    public function actualitza_usuari()
    {   
        $this->load->library('form_validation');
        
        // Validació formulari ok
        if ($this->form_validation->run('admin/usuaris/actualitza_usuari') !== FALSE)
        {
            if ($this->input->post('rol')!=='1' && !$this->input->post('departament'))
            {
                $this->editar_usuari($this->input->post('id_usuari'), 'Has d\'especificar un departament');
            }
            else
            {
                // Desar usuari
                $updated = $this->usuari_model->upd_usuari($this->input->post());
                // Update ok
                if ($updated)
                {
                    $this->messages->add("Usuari actualitzat correctament", 'success', TRUE);
                    redirect('admin/usuaris');
                }
                // Update error
                else
                {
                    $this->editar_usuari($this->input->post('id_usuari'), "Error guardant les dades a la base de dades..");
                }
            }
        }
        // Validació formulari error
        else
        {
            $this->editar_usuari($this->input->post('id_usuari'));
        }
    }
    
    /**
     * Reiniciar la contrasenya de l'usuari enviat per post i se l'avisa per
     * correu electrònic.
     */
    public function reiniciar_contrasenya()
    {
        $this->load->helper('string');
        $this->load->library('phpass');
        
        $contrasenya = strtolower(random_string('alnum', 6));
        $updated = $this->usuari_model->update($this->input->post('id_usuari'), array('contrasenya' => $this->phpass->hash($contrasenya)));
        if ($updated)
        {
            $this->_mail_nova_contrasenya($this->input->post('nom'), $this->input->post('id_usuari'), $contrasenya, $this->input->post('email'));
            $this->messages->add("S'ha reiniciat la contrasenya i enviat per mail a l'usuari".$contrasenya, 'success', TRUE);
            echo json_encode(array('status' => 'ok'));
        }
        else
        {
            echo json_encode(array('status' => 'error', 'error' => "No s'ha pogut desar la contrasenya."));
        }
    }
    
    /**
     * Envia correu electrònic a l'usuari amb una nova contrasenya
     * @param {string} $nom
     * @param {string} $usuari
     * @param {string} $contrasenya
     * @param {string} $email
     */
    private function _mail_nova_contrasenya($nom, $usuari, $contrasenya, $email)
    {
        $this->load->library('email');
        
        $this->data['nom'] = $nom;
        $this->data['usuari'] = $usuari;
        $this->data['contrasenya'] = $contrasenya;
        $this->data['link'] = site_url();
        $this->email->from($this->session->userdata('usuari')->email,
                            $this->session->userdata('usuari')->nom.' '.$this->session->userdata('usuari')->cognoms);
        $this->email->to($email);
        $this->email->subject('['.config_item('app_name').']'.' Nova contrasenya CIAU');
        $this->email->message( $this->load->view('mail/mail_nova_contrasenya', $this->data, true) );
        $this->email->send();
    }
    
    /**
     * Valida nom i cognoms de l'usuari (alpha i espais). S'utilitza a la llibreria
     * form_validation de CodeIgniter
     * @param {string} $str Cadena a validar
     * @return {boolean}
     */
    function _alpha_dash_space($str)
    {
        return ( ! preg_match("/^([-a-z_ ])+$/i", $str)) ? FALSE : TRUE;
    } 

}

/* Fi del fitxer usuaris.php */
/* Ubicació: ./application/controllers/usuaris.php */
