<?php 
/**
 * Controlador autenticació usuaris
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Auth extends CI_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();
        $this->load->model('usuari_model');
    }
    
    /**
     * Redirecciona l'usuari segons està autenticat o no
     */
    public function index()
    {
        if (!$this->session->userdata('loggedin'))
        {
            redirect('auth/login');
        }
        else
        {
            redirect($this->session->userdata('usuari')->rol.'/inici');
        }
    }
    
    /**
     * Mostra pantalla login
     */
    public function login()
    {
        $this->load->view('login', $this->data);
    }
    
    /**
     * Verifica usuari/contrasenya entrades
     */
    public function autenticar()
    {
        $this->load->library('form_validation');
        
        /* Validació formulari ok */
        if ($this->form_validation->run() !== FALSE)
        {
            $usuari = $this->usuari_model->auth_usuari($this->input->post('usuari'), $this->input->post('contrasenya'));
            if (is_object($usuari))
            {
                $this->_set_dades_sessio($usuari);
                
                // En cas d'haver accedir a un controlador en concret se'l redirigeix directament
                if ($this->session->userdata('referrer'))
                {
                    redirect($this->session->userdata('referrer'));
                }
                
                $this->messages->add("Benvingut/da <strong>{$usuari->nom}</strong>!", 'success', TRUE);
            }
            elseif ($usuari === TRUE)
            {
                $this->messages->add('<strong>La contrasenya és incorrecte</strong>', 'error', FALSE);
            }
            elseif ($usuari === FALSE)
            {
                $this->messages->add('<strong>L\'usuari és inexistent o inactiu</strong>', 'error', FALSE);
            }
        }
        /* Validació formulari error */
        else
        {
            $this->messages->add(validation_errors(), 'error', FALSE);
        }
        
        $this->session->set_userdata('usuari_login', $this->input->post('usuari'));
        redirect('/');
    }
    
    /**
     * Esborra les dades de sessió usuari i mostra pantalla login
     */
    public function logout()
    {
        $this->session->sess_destroy();
        redirect('/');
    }
    
    /**
     * Assigna dades de l'usuari a la variable sessió
     * @param {array} $usuari
     */
    private function _set_dades_sessio($usuari)
    {
        $this->session->set_userdata('usuari_login', '');
        $this->session->set_userdata('loggedin', TRUE);
        $this->session->set_userdata('es_'.$usuari->rol, TRUE);
        if (empty($usuari->foto)) $usuari->foto = 'generica.png';
        $this->session->set_userdata('usuari', $usuari);
    }
    
}

/* Fi del fitxer auth.php */
/* Ubicació: ./application/controllers/auth.php */