<?php 
/**
 * Controlador incidències del rol gestor i administrador
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Incidencies_gestor extends Gestor_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();
        
        $this->load->model('incidencia_model');
        $this->load->model('usuari_model');
        $this->load->model('departament_model');
    }

    public function index()
    {
        $this->historic();
    }
    
    /**
     * Carrega les dades d'una incidència i les passa a la vista corresponent
     * @param {string} $id_incidencia ID incidència
     */
    public function veure_incidencia($id_incidencia)
    {
        $this->data['incidencia'] = $this->incidencia_model->get_incidencia($id_incidencia);
        // Sense ID incidència o ID no trobada
        if (!$this->data['incidencia'])
        {
            $this->messages->add("No s'ha especificat un nº d'incidència vàlid", 'error', TRUE);
            redirect('/');
        }
        
        $this->data['departaments'] = $this->departament_model->get_departaments();
        unset($this->data['departaments']['']);
        $this->data['gestors'] = $this->usuari_model->get_gestors_departament($this->data['incidencia']->departament_assignat);

        $this->data['subview'] = 'gestor/incidencies/veure_incidencia';
        $this->data['page_title'] = 'Incidència '.$id_incidencia;
        $this->data['extraHead'] = '<script type="text/javascript" src="'.asset_url().'js/bootbox.min.js"></script>';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Ajax: Retorna gestors d'un departament
     * @param {string} $departament
     */
    public function get_gestors_departament($departament)
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }
        header('Content-Type: application/x-json; charset=utf-8');
        echo(json_encode( $this->usuari_model->get_gestors_departament($departament) ));
    }
    
    /**
     * Carrega les actualitzacions de la incidència corresponent i les mostra
     * en una taula.
     * @param {string} $id_incidencia
     */
    public function veure_taula_actualitzacions($id_incidencia)
    {
        $actualitzacions = $this->incidencia_model->get_actualitzacions($id_incidencia);
        
        if ($actualitzacions)
        {
            $this->load->library('table');
            $tmpl = array ( 'table_open'  => '<table class="table table-striped">' );
            $this->table->set_template($tmpl);
            $this->table->set_heading('Data', 'Usuari', 'Tipus', 'Canvi');
            echo $this->table->generate($this->incidencia_model->get_actualitzacions($id_incidencia));
        }
        else
        {
            echo "<h4 class='center'>Encara no s'ha realitzat cap actualització d'estat, prioritat o assignació</h4>";
        }
    }
    
    /**
     * Ajax: Guarda el canvi de departament i gestor assignat d'una incidència
     */
    public function guarda_canvi_assignacio()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }
        
        $this->load->model('estat_model');
        
        $incidencia = $this->incidencia_model->get($this->input->post('incidencia'));

        if ($incidencia->gestor_assignat !== $this->input->post('gestor') ||
                $incidencia->departament_assignat !== $this->input->post('departament'))
        {
            $assignacio_antiga = $this->departament_model->get($incidencia->departament_assignat)->nom .' / '.
                                $incidencia->gestor_assignat;
            $estat = $this->incidencia_model->upd_assignacions($this->input->post('incidencia'),
                $assignacio_antiga,
                $this->input->post('departament'),
                $this->departament_model->get($this->input->post('departament'))->nom,
                $this->input->post('gestor'),
                $incidencia->estat);
            $resultat = array('status' => 'ok', 'estat' => $this->estat_model->get($estat)->nom);
        }
        else
        {
            $resultat = array('status' => 'error', 'error' => 'El departament i gestor no han canviat');
        }
        
        echo json_encode($resultat);
    }
    
    /**
     * Ajax: Guarda el canvi de prioritat d'una incidència
     */
    public function guarda_canvi_prioritat()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }
        
        $incidencia = $this->incidencia_model->get($this->input->post('incidencia'));

        if ($incidencia->prioritat !== $this->input->post('prioritat'))
        {
            $this->incidencia_model->upd_prioritat($this->input->post('incidencia'),
                $incidencia->prioritat,
                $this->input->post('prioritat'));
            $resultat = array('status' => 'ok');
        }
        else
        {
            $resultat = array('status' => 'error', 'error' => 'La prioritat no han canviat');
        }
        
        echo json_encode($resultat);
    }
    
    /**
     * Ajax: guarda una nova nota de seguiment
     */
    public function guarda_nota_seguiment()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }

        $this->load->model('notaseguiment_model');
        $this->load->library('form_validation');
        
        /* Validació formulari error */
        if ($this->form_validation->run('gestor/incidencies/guarda_nota_seguiment') === FALSE)
        {
            echo json_encode(array('status' => 'error', 'error' => validation_errors()));
        }
        /* Validació formulari ok */
        else
        {
            // Si és un feedback avisa per correu al sol·licitant
            if ($this->input->post('tipus_nota') === 'feedback')
            {
                $solicitant = $this->usuari_model->get_usuari($this->input->post('solicitant'));
                $this->_mail_solicitant_feedback($solicitant, $this->input->post('incidencia'), $this->input->post('enespera'));
                $resultat['mail'] = 'enviat';
            }
            
            // Insereix a BD
            $insert_data = array(
                'incidencia' => $this->input->post('incidencia'),
                'tipus' => $this->input->post('tipus_nota'),
                'descripcio' => nl2br($this->input->post('nota'))
            );
            $this->notaseguiment_model->insert($insert_data);
            $resultat = array('status' => 'ok');
            
            // Passar a "En Espera"
            if ($this->input->post('tipus_nota') === 'feedback' &&
                $this->input->post('enespera') === "si")
            {
                $this->incidencia_model->upd_estat($this->input->post('incidencia'), $this->input->post('estat'), 3);
                $resultat['estat_des'] = 'En espera';
                $resultat['estat'] = '3';
            }
            
            echo json_encode($resultat);
        }
    }
    
    /**
     * Envia correu electrònic al sol·licitant avisant nova nota seguiment
     * @param {object} $solicitant Dades del sol·licitant
     * @param {string} $incidencia ID Incidència
     * @param {string} $enespera Indica si la incidència ha quedat en estat "En espera"
     */
    private function _mail_solicitant_feedback($solicitant, $incidencia, $enespera)
    {
        $this->load->library('email');
        $this->data['nom'] = $solicitant->nom;
        $this->data['incidencia'] = $incidencia;
        $this->data['enespera'] = $enespera;
        $this->data['link'] = base_url('solicitant/incidencies/veure_incidencia/'.$incidencia.'#notes');
        $this->email->from($this->session->userdata('usuari')->email,
                            $this->session->userdata('usuari')->nom.' '.$this->session->userdata('usuari')->cognoms);
        $this->email->to($solicitant->email);
        $this->email->subject('[Incidència '.$incidencia.'] Nova nota de seguiment');
        $this->email->message( $this->load->view('mail/mail_solicitant_feedback', $this->data, true) );
        $this->email->send();
    }
    
    /**
     * Ajax: guarda una solució
     */
    public function guarda_solucio()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }

        $this->load->model('notaseguiment_model');
        $this->load->library('form_validation');
        
        /* Validació formulari error */
        if ($this->form_validation->run('solicitant/incidencies/guarda_nota_seguiment') === FALSE)
        {
            echo json_encode(array('status' => 'error', 'error' => validation_errors()));
        }
        /* Validació formulari ok */
        else
        {
            $incidencia = $this->incidencia_model->get_incidencia($this->input->post('incidencia'));
            if ($incidencia->gestor_assignat)
            {
                // Avisa per correu al sol·licitant
                $solicitant = $this->usuari_model->get_usuari($this->input->post('solicitant'));
                $this->_mail_solicitant_solucio($solicitant, $this->input->post('incidencia'));

                // Insereix a BD
                $insert_data = array(
                    'incidencia' => $this->input->post('incidencia'),
                    'tipus' => 'solucio',
                    'descripcio' => nl2br($this->input->post('nota'))
                );
                $this->notaseguiment_model->insert($insert_data);
                $resultat = array('status' => 'ok');

                // Passar a "Tancada"
                $this->incidencia_model->upd_estat($this->input->post('incidencia'), $this->input->post('estat'), 4);

                $this->messages->add("Incidència tancada correctament", 'success', TRUE);
                echo json_encode($resultat);
            }
            else
            {
                echo json_encode(array('status' => 'error', 'error' => 'Abans de tancar una incidència cal assignar-li un gestor'));
            }
        }
    }
    
    /**
     * Envia correu electrònic al sol·licitant avisant del tancament de la incidència
     * @param {object} $solicitant Dades del sol·licitant
     * @param {string} $incidencia ID Incidència
     */
    private function _mail_solicitant_solucio($solicitant, $incidencia)
    {
        $this->load->library('email');
        $this->data['nom'] = $solicitant->nom;
        $this->data['incidencia'] = $incidencia;
        $this->data['link'] = base_url('solicitant/incidencies/veure_incidencia/'.$incidencia.'#solucio');
        $this->email->from($this->session->userdata('usuari')->email,
                            $this->session->userdata('usuari')->nom.' '.$this->session->userdata('usuari')->cognoms);
        $this->email->to($solicitant->email);
        $this->email->subject('[Incidència '.$incidencia.'] Incidència tancada');
        $this->email->message( $this->load->view('mail/mail_solicitant_solucio', $this->data, true) );
        $this->email->send();
    }
    
    /**
     * Mostra totes les incidències assignades i permet filtrar-les segons criteris
     */
    public function historic()
    {
        $this->load->model('motiu_model');
        $this->load->model('estat_model');
        
        $filtres = array();
        $filtres['text'] = $this->input->post('text');
        $filtres['prioritat'] = $this->input->post('prioritat');
        $filtres['motiu'] = $this->input->post('motiu');
        $filtres['solicitant'] = $this->input->post('solicitant');
        $filtres['gestor'] = $this->input->post('gestor');
        $filtres['departament'] = $this->input->post('departament');
        if ($this->input->post('estat'))
            $filtres['estats'] = array($this->input->post('estat'));
        if ($this->input->post('desde'))
            $filtres['desde'] = date("Y-m-d", strtotime(str_replace('/', '-', $this->input->post('desde'))));
        if ($this->input->post('fins'))
            $filtres['fins'] = date("Y-m-d", strtotime(str_replace('/', '-', $this->input->post('fins'))));
        
        $this->load->library('pagination');
        $config['base_url'] = base_url().'gestor/incidencies/historic';
        $config['per_page'] = '10';
        $config['uri_segment'] = '4';
        $config['total_rows'] = count($this->incidencia_model->get_incidencies($filtres));
        $this->pagination->initialize($config);
        $inici = ($this->uri->segment(4)) ? $this->uri->segment(4) : 0;
        
        $this->data['incidencies'] = $this->incidencia_model->get_incidencies($filtres, $config['per_page'], $inici);
        $this->data['incidencies_total'] = $config['total_rows'];
        $this->data['paginacio'] = $this->pagination->create_links();
        $this->data['motius'] = $this->motiu_model->get_motius();
        $this->data['estats'] = $this->estat_model->get_estats();
        $this->data['departaments'] = $this->departament_model->get_departaments();
        $this->data['gestors'] = $this->usuari_model->get_gestors_departament();
        
        $this->data['extraHead'] = '<link href="'.asset_url().'css/bootstrap-rowlink.min.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-rowlink.min.js"></script>
                                    <link href="'.asset_url().'css/bootstrap-datepicker.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-datepicker.js"></script>
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-datepicker-locales/bootstrap-datepicker.ca.js"></script>';
        $this->data['page_title'] = 'Històric';
        $this->data['subview'] = 'gestor/incidencies/historic';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Ajax: formata dades usuaris segons el seu rol i nom per mostrar a una select
     */
    public function autocomplete_usuari()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }
        echo json_encode($this->usuari_model->get_usuaris_autocomplete($this->input->get('rol')));
    }

}

/* Fi del fitxer incidencies.php */
/* Ubicació: ./application/controllers/gestor/incidencies.php */
