<?php 
/**
 * Controlador preguntes freqüents dels rols gestor i administrador
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Pf_gestor extends Gestor_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();	
        $this->load->model('pf_model');
    }

    /**
     * Llistat i filtre de preguntes freqüents, carrega vista principal
     */
    public function index()
    {   
        $filtres = array();
        $filtres['text'] = $this->input->post('text');
        
        $this->load->library('pagination');
        $config['base_url'] = site_url('gestor/pf');
        $config['per_page'] = '15';
        $config['uri_segment'] = '3';
        $config['total_rows'] = count($this->pf_model->get_pfs($filtres));
        $this->pagination->initialize($config);
        $inici = ($this->uri->segment(3)) ? $this->uri->segment(3) : 0;
        
        $this->data['pfs'] = $this->pf_model->get_pfs($filtres, $config['per_page'], $inici);
        $this->data['pfs_total'] = $config['total_rows'];
        $this->data['paginacio'] = $this->pagination->create_links();
        
        $this->data['extraHead'] = '<link href="'.asset_url().'css/bootstrap-rowlink.min.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-rowlink.min.js"></script>';
        $this->data['page_title'] = 'Preguntes freqüents';
        $this->data['subview'] = 'gestor/pf/pf';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Carrega vista Nova PF
     * @param {string} $error Error a mostrar a la vista
     */
    public function nova_pf($error = '')
    {
        $this->load->model('motiu_model');
        
        $this->data['error'] = $error;
        $this->data['motius'] = $this->motiu_model->get_motius();
        unset($this->data['motius']['']);
        $this->data['page_title'] = 'Nova pregunta freqüent';
        $this->data['subview'] = 'gestor/pf/nova_pf';
        $this->load->view('template/layout', $this->data);
    }

    /**
     * Valida i guarda l'usuari enviat
     */
    public function guarda_pf()
    {   
        $this->load->library('form_validation');
        
        // Validació formulari ok
        if ($this->form_validation->run('gestor/pf/guarda_pf') !== FALSE)
        {
            if (!$this->input->post('motiu'))
            {
                $this->nova_pf('Has d\'especificar algun motiu relacionat');
            }
            else
            {
                // Desar pf
                $insert = $this->pf_model->insert_pf($this->input->post());
                // Insert ok
                if ($insert)
                {
                    $this->messages->add("Pregunta freqüent creada correctament", 'success', TRUE);
                    redirect('gestor/pf');
                }
                // Insert error
                else
                {
                    $this->nova_pf("Error guardant les dades a la base de dades..");
                }
            }
        }
        // Validació formulari error
        else
        {
            $this->nova_pf();
        }
    }
    
    /**
     * Carrega vista Editar PF
     * @param {string} $id_pf Identificador PF a editar
     * @param {string} $error Error a mostrar a la vista
     */
    public function editar_pf($id_pf, $error = '')
    {
        $this->data['pf'] = $this->pf_model->get_pf($id_pf);
        // Sense Id PF o PF no trobada
        if (!$this->data['pf'])
        {
            $this->messages->add("No s'ha especificat una pregunta freqüent vàlida", 'error', TRUE);
            redirect('gestor/pf');
        }
        $this->data['motius_relacionats'] = $this->pf_model->get_motius_relacionats($id_pf);
        
        $this->load->model('motiu_model');
        $this->data['motius'] = $this->motiu_model->get_motius();
        unset($this->data['motius']['']);
        
        $this->data['error'] = $error;
        $this->data['extraHead'] = '<script type="text/javascript" src="'.asset_url().'js/bootbox.min.js"></script>';
        $this->data['page_title'] = 'Editar pregunta freqüent '.$id_pf;
        $this->data['subview'] = 'gestor/pf/editar_pf';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Valida i guarda la PF enviada
     */
    public function actualitza_pf()
    {   
        $this->load->library('form_validation');
        
        // Validació formulari ok
        if ($this->form_validation->run('gestor/pf/guarda_pf') !== FALSE)
        {
            if (!$this->input->post('motiu'))
            {
                $this->editar_pf($this->input->post('id_preguntaFrequent'), 'Has d\'especificar algun motiu relacionat');
            }
            else
            {
                // Desar PF
                $updated = $this->pf_model->upd_pf($this->input->post());
                // Update ok
                if ($updated)
                {
                    $this->messages->add("Pregunta freqüent actualitzada correctament", 'success', TRUE);
                    redirect('gestor/pf');
                }
                // Update error
                else
                {
                    $this->editar_pf($this->input->post('id_preguntaFrequent'), "Error guardant les dades a la base de dades..");
                }
            }
        }
        // Validació formulari error
        else
        {
            $this->editar_pf($this->input->post('id_preguntaFrequent'));
        }
    }
    
    /**
     * Esborra pf de la base de dades
     */
    public function esborra_pf()
    {
        $deleted = $this->pf_model->delete_pf($this->input->post('id_preguntaFrequent'));
        if ($deleted)
        {
            $this->messages->add("S'ha esborrat la pregunta freqüent", 'success', TRUE);
            echo json_encode(array('status' => 'ok'));
        }
        else
        {
            echo json_encode(array('status' => 'error', 'error' => "No s'ha pogut esborrar."));
        }
    }
}

/* Fi del fitxer pf.php */
/* Ubicació: ./application/controllers/gestor/pf.php */
