<?php 
/**
 * Controlador global incidències, funcions comunes usuaris
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Incidencies extends CI_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();
        
        if (!$this->session->userdata('loggedin')) {
            $this->session->set_userdata('referrer', current_url());
            redirect('/');
        }
    }

    /**
     * Ajax: Carrega les notes de següiment d'una incidència i les
     * passa a la vista corresponent. Retorna només les corresponents segons el
     * rol de l'usuari autenticat.
     * @param {string} $id_incidencia ID incidència
     */
    public function veure_notes_seguiment($id_incidencia)
    {
        $this->load->model('notaseguiment_model');
        $this->data['notes_seguiment'] = $this->notaseguiment_model->get_notes_seguiment($id_incidencia);
        $this->load->view('ajax_notes_seguiment', $this->data);
    }
    
    /**
     * Ajax: Carrega les solucions d'una incidència i les passa a la vista corresponent.
     * @param {string} $id_incidencia ID incidència
     */
    public function veure_solucions($id_incidencia)
    {
        $this->load->model('notaseguiment_model');
        $this->data['solucions'] = $this->notaseguiment_model->get_notes_seguiment($id_incidencia, array('solucio'));
        $this->load->view('ajax_solucions', $this->data);
    }
    
    /**
     * Ajax: torna a obrir incidència
     */
    public function reobre_incidencia()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }
        
        $this->load->model('incidencia_model');
        
        $incidencia = $this->incidencia_model->get($this->input->post('incidencia'));
        
        // Actualitza estat
        if ($incidencia->estat === "4" && time() <= strtotime($incidencia->updated_on) + config_item('hores_reobre_incidencia')*3600)
        {
            $this->incidencia_model->upd_ultim_estat($this->input->post('incidencia'), $incidencia->estat);
            $resultat = array('status' => 'ok');
        }
        else
        {
            $resultat = array('status' => 'error', 'error' => 'Incidència no tancada recentment (72h)');
        }
        
        $this->messages->add("Incidència reoberta correctament", 'success', TRUE);
        echo json_encode($resultat);
    }

}

/* Fi del fitxer incidencies.php */
/* Ubicació: ./application/controllers/incidencies.php */