<?php 
/**
 * Controlador perfil usuaris
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Perfil extends CI_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();	
        if (!$this->session->userdata('loggedin'))
        {
            $this->session->set_userdata('referrer', current_url());
            redirect('/');
        }
        $this->load->model('usuari_model');
        $this->load->model('verificaemail_model');
    }

    /**
     * Carrega vista Perfil
     */
    public function index()
    {
        $this->load->model('incidencia_model');
        
        // Comprova que no tingui algun canvi de correu electrònic pendent, i l'avisa
        $res = $this->verificaemail_model->get($this->session->userdata('usuari')->id_usuari);
        if (count($res)>0)
        {
            $interval = date_diff(new DateTime($res->added_on), new DateTime(date("Y-m-d H:i:s")));
            if ($interval->days === 0)
            {
                $this->messages->add('Tens un canvi de correu electrònic pendent ('.$res->nou_email.')', 'info', FALSE);
            }
        }
        
        $this->data['incidencies_total'] = $this->incidencia_model->count_by('solicitant', $this->session->userdata('usuari')->id_usuari);
        $this->data['incidencies_obertes'] = $this->incidencia_model->count_by(
                array('solicitant' => $this->session->userdata('usuari')->id_usuari,
                    'estat !=' => 4));
        $this->data['usuari'] = $this->session->userdata('usuari');
        $this->data['subview'] = 'perfil';
        $this->data['page_title'] = 'Perfil '.$this->session->userdata('usuari')->id_usuari;
        $this->data['extraHead'] = '<script type="text/javascript" src="'.asset_url().'js/ajaxfileupload.js"></script>
                                    <link href="'.asset_url().'css/bootstrap-fileupload.min.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-fileupload.min.js"></script>';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Ajax: Valida i guarda la foto de perfil seleccionada
     */
    public function canvi_foto()
    {        
        $this->config->load('upload_foto', TRUE); // config/upload_foto
        $config = $this->config->item('upload_foto');
        $config['file_name'] = $this->session->userdata('usuari')->id_usuari;    
        $this->load->library('upload', $config); 
        
        /* Upload error */
        if (!$this->upload->do_upload('foto'))
        {
            echo $this->upload->display_errors();
        }
        /* Upload ok */
        else
        {
            // Dades sessió
            $usuari = $this->session->userdata('usuari');
            $upload = $this->upload->data();
            if (strtolower($usuari->foto) !== strtolower($upload['file_name']))
            {
                if ($usuari->foto !== 'generica.png')
                {
                    unlink('./assets/uploads/fotos_perfil/'.$usuari->foto);
                }
                $usuari->foto = $upload['file_name'];
                $this->session->set_userdata('usuari', $usuari);
            }
            
            // Actualitza BD
            $this->usuari_model->update($this->session->userdata('usuari')->id_usuari,
                                        array('foto' => $upload['file_name'], 'updated_by' => $this->session->userdata('usuari')->id_usuari));
             
            // Feedback
            echo json_encode(array('status' => 'ok', 'foto' => $upload['file_name']));
        }
    }
    
    /**
     * Ajax: actualitza la contrasenya a la base de dades
     */
    public function canvi_contrasenya()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }
        
        $this->load->library('form_validation'); // config/form_validation
        
        /* Validació formulari error */
        if ($this->form_validation->run() === FALSE)
        {
            echo json_encode(array('status' => 'error', 'error' => validation_errors()));
        }
        /* Validació formulari ok */
        else
        {
            $usuariAutenticat = $this->usuari_model->auth_usuari($this->session->userdata('usuari')->id_usuari,
                                                                $this->input->post('contrasenya'));
            if (!is_object($usuariAutenticat))
            {
                // Actualitza BD
                $this->load->library('phpass');
                $this->usuari_model->update($this->session->userdata('usuari')->id_usuari,
                                            array('contrasenya' => $this->phpass->hash($this->input->post('contrasenya')),
                                                    'updated_by' => $this->session->userdata('usuari')->id_usuari));
                echo json_encode(array('status' => 'ok'));
            }
            else
            {
                echo json_encode(array('status' => 'error', 'error' => 'La contrasenya és igual a l\'anterior'));
            }
        }
    }
    
    /**
     * Ajax: envia mail per confirmar canvi adreça electrònica
     */
    public function canvi_mail()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }
        
        $this->load->library('form_validation'); // config/form_validation
        
        /* Validació formulari error */
        if ($this->form_validation->run() === FALSE)
        {
            echo json_encode(array('status' => 'error', 'error' => validation_errors()));
        }
        /* Validació formulari ok */
        else
        {
            if ($this->session->userdata('usuari')->email !== $this->input->post('mail'))
            {
                $codi_verificacio = random_string('alnum', 16);
                $this->verificaemail_model->delete($this->session->userdata('usuari')->id_usuari);
                $this->verificaemail_model->insert(array('usuari'=>$this->session->userdata('usuari')->id_usuari,
                                                            'nou_email'=>$this->input->post('mail'),
                                                            'codi_verificacio'=>$codi_verificacio));
                $this->_mail_canvi_mail($codi_verificacio);
                echo json_encode(array('status' => 'ok'));
            }
            else
            {
                echo json_encode(array('status' => 'error', 'error' => 'El nou correu ha de ser diferent a l\'anterior'));
            }
        }
    }
    
    /**
     * Envia correu electrònic a la nova adreça amb codi verificació
     * @param {string} $codi_verificacio
     */
    private function _mail_canvi_mail($codi_verificacio)
    {
        $this->data['nom'] = $this->session->userdata('usuari')->nom;
        $this->data['mail'] = $this->input->post('mail');
        $this->data['link'] = base_url('perfil/confirma_canvi_mail/'.$codi_verificacio);
        $this->load->library('email');
        $this->email->from(config_item('admin_mail'), 'Admin');
        $this->email->to($this->input->post('mail'));
        $this->email->subject('['.config_item('app_name').']'.' Confirmació canvi correu electrònic');
        $this->email->message( $this->load->view('mail/mail_canvi_mail', $this->data, true) );
        $this->email->send();
    }
    
    /**
     * Comprova que el codi sigui correcte i actualitza el mail.
     * S'arriba a aquest controlador clicant l'enllaç del mail de confirmació rebut
     * @param {string} $codi_verificacio
     */
    public function confirma_canvi_mail($codi_verificacio)
    {
        $res = $this->verificaemail_model->get_by(array('usuari' => $this->session->userdata('usuari')->id_usuari,
                                                        'codi_verificacio' => $codi_verificacio));
        if (count($res)>0)
        {
            $interval = date_diff(new DateTime($res->added_on), new DateTime(date("Y-m-d H:i:s")));
            if ($interval->days === 0)
            {
                // Canvia dades sessió
                $usuari = $this->session->userdata('usuari');
                $usuari->email = $res->nou_email;
                $this->session->set_userdata('usuari', $usuari);
                // Actualitza BD
                $this->usuari_model->update($this->session->userdata('usuari')->id_usuari,
                                            array('email' => $res->nou_email));
                $this->verificaemail_model->delete($this->session->userdata('usuari')->id_usuari);
                // Mostra confirmació
                $this->messages->add('Correu electrònic actualitzat correctament', 'success', TRUE);
            }
            else
            {
                $this->messages->add('El canvi havia expirat, cal tornar a demanar-lo', 'error', TRUE);
            }
        }
        else
        {
            $this->messages->add('Codi de verificació incorrecte', 'error', TRUE);
        }
        redirect('/perfil');
    }

}

/* Fi del fitxer perfil.php */
/* Ubicació: ./application/controllers/perfil.php */