<?php 
/**
 * Controlador incidències del rol sol·licitant
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Incidencies_solicitant extends Solicitant_Controller
{

    public $data = array('subview' => 'Ops, no s\'ha especificat cap subview');
    
    public function __construct()
    {
        parent::__construct();
        
        $this->load->model('motiu_model');
        $this->load->model('incidencia_model');
        $this->load->model('estat_model');
        $this->load->model('notaseguiment_model');
    }

    /**
     * Per defecte el controlador incidència mostra l'històric
     */
    public function index()
    {
        $this->historic();
    }
    
    /**
     * Carrega les dades d'una incidència i les passa a la vista corresponent
     * @param {string} $id_incidencia ID incidència
     */
    public function veure_incidencia($id_incidencia)
    {
        $this->data['incidencia'] = $this->incidencia_model->get_incidencia($id_incidencia);
        
        // Sense ID incidència o ID no trobada
        if (!$this->data['incidencia'])
        {
            $this->messages->add("No s'ha especificat un nº d'incidència vàlid", 'error', TRUE);
            redirect('/');
        }
        // Un sol·licitant només pot veure les seves incidències
        elseif ($this->data['incidencia']->solicitant !== $this->session->userdata('usuari')->id_usuari)
        {
            $this->messages->add('Accés denegat', 'error', TRUE);
            redirect('/');
        }
        
        // Carrega les solucions
        list($this->data['solucions'], $pendent_valoracio) = $this->_get_solucions($id_incidencia);
        if ($pendent_valoracio)
        {
            $this->messages->add('La solució a la incidència està pendent de valoració', 'info', FALSE);
        }

        $this->data['subview'] = 'solicitant/incidencies/veure_incidencia';
        $this->data['page_title'] = 'Incidència '.$id_incidencia;
        $this->data['extraHead'] = '<script type="text/javascript" src="'.asset_url().'js/bootbox.min.js"></script>';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Carrega les dades de les solucions d'una incidència
     * @param {string} $id_incidencia ID incidència
     * @return {array} Com a primer valor les solucions i com a segon valor un
     *                  boleà indicant si estan pendents de valoració
     */
    private function _get_solucions($id_incidencia)
    {
        $solucions = $this->notaseguiment_model->get_notes_seguiment($id_incidencia, array('solucio'));
        $pendent_valoracio = FALSE;
        foreach ($solucions as $solucio)
        {
            if ($solucio->valoracio === null) $pendent_valoracio = TRUE;
        }
        return array($solucions, $pendent_valoracio);
    }
       
    /**
     * Carrega vista Nova Incidència
     * @param {string} $error Error a mostrar a la vista
     */
    public function nova_incidencia($error = '')
    {
        $this->data['error'] = $error;
        $this->data['motius'] = $this->motiu_model->get_motius();
        $this->data['usuari'] = $this->session->userdata('usuari');
        $this->data['extraHead'] = '<link href="'.asset_url().'css/bootstrap-fileupload.min.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-fileupload.min.js"></script>';
        $this->data['page_title'] = 'Nova incidència';
        $this->data['subview'] = 'solicitant/incidencies/nova_incidencia';
        $this->load->view('template/layout', $this->data);
    }
    
    /**
     * Valida i guarda la incidència enviada
     */
    public function guarda_incidencia()
    {     
        if (!$this->input->post('prioritat')) redirect('solicitant/incidencies/nova_incidencia');
        
        $this->load->library('form_validation');
        // Validació formulari ok
        if ($this->form_validation->run('solicitant/incidencies/guarda_incidencia') !== FALSE)
        {
            // Amb fitxer adjunt
            $adjunt = '';
            if (!empty($_FILES['adjunt']['tmp_name']))
            {
                $this->config->load('upload_fitxer', TRUE);
                $config = $this->config->item('upload_fitxer');
                $config['file_name'] = time().'_'.$_FILES['adjunt']['name'];
                $this->load->library('upload', $config);
                
                // Upload ok
                if ($this->upload->do_upload('adjunt'))
                {
                    $adjunt = $this->upload->file_name;
                }
                // Upload error
                else
                {
                    $this->nova_incidencia($this->upload->display_errors('<span>','<span>'));
                }
            }

            // Desar incidència
            if (empty($_FILES['adjunt']['tmp_name']) || ($adjunt))
            {
                $insert = $this->incidencia_model->insert_incidencia($this->input->post(), $adjunt);
                // Insert ok
                if ($insert)
                {
                    $this->messages->add("Incidència creada correctament", 'success', TRUE);
                    redirect('solicitant/incidencies/veure_incidencia/'.$insert);
                }
                // Insert error
                else
                {
                    $this->nova_incidencia("Error guardant les dades a la base de dades..");
                }
            }
        }
        // Validació formulari error
        else
        {
            $this->nova_incidencia();
        }
    }
    
    /**
     * Ajax: guarda una nova nota de seguiment
     */
    public function guarda_nota_seguiment()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }

        $this->load->model('usuari_model');
        $this->load->library('form_validation');
        /* Validació formulari error */
        if ($this->form_validation->run('solicitant/incidencies/guarda_nota_seguiment') === FALSE)
        {
            echo json_encode(array('status' => 'error', 'error' => validation_errors()));
        }
        /* Validació formulari ok */
        else
        {
            // Insereix a BD
            $insert_data = array(
                'incidencia' => $this->input->post('incidencia'),
                'tipus' => 'solicitant',
                'descripcio' => nl2br($this->input->post('nota'))
            );
            $this->notaseguiment_model->insert($insert_data);
            $resultat = array('status' => 'ok');
            
            // Si cal, actualitza estat
            if ($this->input->post('estat') == "3")
            {
                $nou_estat = $this->incidencia_model->upd_ultim_estat($this->input->post('incidencia'),
                                                                        $this->input->post('estat'));
                $resultat['estat'] = $nou_estat;
                $resultat['estat_des'] = $this->estat_model->get($nou_estat)->nom;
            }
            
            // Si té gestor assignat avisa per correu
            if ($this->input->post('gestor_assignat'))
            {
                $gestor = $this->usuari_model->get_usuari($this->input->post('gestor_assignat'));
                $this->_mail_gestor_notaseguiment($gestor, $this->input->post('incidencia'));
                $resultat['mail'] = 'enviat';
            }

            echo json_encode($resultat);
        }
    }
    
    /**
     * Envia correu electrònic al gestor avisant nova nota seguiment
     * @param {object} $gestor Dades del gestor
     * @param {string} $incidencia ID Incidència
     */
    private function _mail_gestor_notaseguiment($gestor, $incidencia)
    {
        $this->load->library('email');
        $this->data['nom'] = $gestor->nom;
        $this->data['incidencia'] = $incidencia;
        $this->data['link'] = base_url('gestor/incidencies/veure_incidencia/'.$incidencia.'#notes');
        $this->email->from($this->session->userdata('usuari')->email,
                            $this->session->userdata('usuari')->nom.' '.$this->session->userdata('usuari')->cognoms);
        $this->email->to($gestor->email);
        $this->email->subject('[Incidència '.$incidencia.'] Nova nota de seguiment');
        $this->email->message( $this->load->view('mail/mail_gestor_notaseguiment', $this->data, true) );
        $this->email->send();
    }
    
    /**
     * Ajax: guarda la valoració d'una solució
     */
    public function valora_solucio()
    {
        if (!$this->input->is_ajax_request())
        {
            exit('Accés denegat');
        }

        // Actualitza valoració
        $this->notaseguiment_model->update($this->input->post('solucio'),
                                            array('valoracio' => $this->input->post('valoracio')));
        
        echo json_encode(array('status' => 'ok'));
    }
    
    /**
     * Mostra totes les incidències de l'usuari i permet filtrar-les segons criteris
     */
    public function historic()
    {
        $filtres = array();
        $filtres['solicitant'] = $this->session->userdata('usuari')->id_usuari;
        $filtres['text'] = $this->input->post('text');
        $filtres['prioritat'] = $this->input->post('prioritat');
        $filtres['motiu'] = $this->input->post('motiu');
        if ($this->input->post('estat'))
            $filtres['estats'] = array($this->input->post('estat'));
        if ($this->input->post('desde'))
            $filtres['desde'] = date("Y-m-d", strtotime(str_replace('/', '-', $this->input->post('desde'))));
        if ($this->input->post('fins'))
            $filtres['fins'] = date("Y-m-d", strtotime(str_replace('/', '-', $this->input->post('fins'))));
        
        $this->load->library('pagination');
        $config['base_url'] = base_url().'solicitant/incidencies/historic';
        $config['per_page'] = '10';
        $config['uri_segment'] = '4';
        $config['total_rows'] = count($this->incidencia_model->get_incidencies($filtres));
        $this->pagination->initialize($config);
        $inici = ($this->uri->segment(4)) ? $this->uri->segment(4) : 0;
        
        $this->data['incidencies'] = $this->incidencia_model->get_incidencies($filtres, $config['per_page'], $inici);
        $this->data['incidencies_total'] = $config['total_rows'];
        $this->data['paginacio'] = $this->pagination->create_links();
        $this->data['motius'] = $this->motiu_model->get_motius();
        $this->data['estats'] = $this->estat_model->get_estats();
        
        $this->data['extraHead'] = '<link href="'.asset_url().'css/bootstrap-rowlink.min.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-rowlink.min.js"></script>
                                    <link href="'.asset_url().'css/bootstrap-datepicker.css" rel="stylesheet">
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-datepicker.js"></script>
                                    <script type="text/javascript" src="'.asset_url().'js/bootstrap-datepicker-locales/bootstrap-datepicker.ca.js"></script>';
        $this->data['page_title'] = 'Històric';
        $this->data['subview'] = 'solicitant/incidencies/historic';
        $this->load->view('template/layout', $this->data);
    }

}

/* Fi del fitxer incidencies.php */
/* Ubicació: ./application/controllers/solicitant/incidencies.php */
