<?php 

if (!defined('BASEPATH')) exit('Accés denegat');

/**
 * Message:: a class for writing feedback message information to the session
 *
 * Copyright 2006 Vijay Mahrra & Sheikh Ahmed <webmaster@designbyfail.com>
 * Copyright 2013 Dimas Streich ('info' instead of 'message', 'output' function
 *                                  and 'fixed' parameter)
 *
 * See the enclosed file COPYING for license information (LGPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @author  1.0 Vijay Mahrra & Sheikh Ahmed | 2.0 Dimas Streich Colomeda
 * @url http://www.designbyfail.com/ & http://www.dimas.cat
 * @version 2.0
 */

class Messages
{
    var $_ci;
    var $_types = array('success', 'error', 'warning', 'info');            

    function Messages($params = array())
    {
        $this->_ci =& get_instance();
        $this->_ci->load->library('session');
        // check if theres already messages, if not, initialise the messages array in the session
        $messages = $this->_ci->session->userdata('messages');
        if (empty($messages)) {
            $this->clear();
        }
    }

    // clear all messages
    function clear()
    {
        $messages = array();
        foreach ($this->_types as $type) {
            $messages[$type] = array();
        }
        $this->_ci->session->set_userdata('messages', $messages);
    }

    // add a message, default type is info
    function add($message, $type = 'info', $fixed = FALSE)
    {
        $messages = $this->_ci->session->userdata('messages');
        // handle PEAR errors gracefully
        if (is_a($message, 'PEAR_Error')) {
            $message = $message->getMessage();
            $type = 'error';
        } else if (!in_array($type, $this->_types)) {
            // set the type to info if the user specified a type that's unknown
            $type = 'info';
        }
        // two fixed messages are not possible
        if ($fixed && $this->recursive_array_search(true,$messages)) {
            trigger_error("You can't add more than 1 fixed missage", E_USER_WARNING);
        }
        
        // don't repeat messages!
        if (!in_array($message, $messages[$type]) && is_string($message)) {
            $message = array ('message' => $message, 'fixed' => $fixed);
            $messages[$type][] = $message;
        }
        $messages = $this->_ci->session->set_userdata('messages', $messages);
    }

    // return messages of given type or all types, return false if none
    function sum($type = null)
    {
        $messages = $this->_ci->session->userdata('messages');
        if (!empty($type)) {
            $i = count($messages[$type]);
            return $i;
        }
        $i = 0;
        foreach ($this->_types as $type) {
            $i += count($messages[$type]);
        }
        return $i;
    }

    // return messages of given type or all types, return false if none, clearing stack
    function get($type = null)
    {
        $messages = $this->_ci->session->userdata('messages');
        if (!empty($type)) {
            if (count($messages[$type]) == 0) {
                return false;
            }
            return $messages[$type];
        }
        // return false if there actually are no messages in the session
        $i = 0;
        foreach ($this->_types as $type) {
            $i += count($messages[$type]);
        }
        if ($i == 0) {
            return false;
        }

        // order return by order of type array above
        // i.e. success, error, warning and then informational messages last
        foreach ($this->_types as $type) {
            $return[$type] = $messages[$type];
        }
        $this->clear();
        return $return;
    }
    
    // return html/css formatted messages
    function output($type = null)
    {
        $messages = $this->_ci->session->userdata('messages');
        if (!empty($type)) {
            if (count($messages[$type]) == 0) {
                return false;
            }
            return $this->makeHtml($messages[$type], $type);
        }
        // return false if there actually are no messages in the session
        $i = 0;
        foreach ($this->_types as $type) {
            $i += count($messages[$type]);
        }
        if ($i == 0) {
            return false;
        }

        // order return by order of type array above
        // i.e. success, error, warning and then informational messages last
        $return = "";
        foreach ($this->_types as $type) {
            if (count($messages[$type]) > 0) $return .= $this->makeHtml($messages[$type], $type);
        }
        $this->clear();
        return $return;
    }
    
    /**
     * 
     * @param Array $messages
     * @param String $type
     * @return html
     * @author Dimas
     */
    function makeHtml($messages, $type) {
        $html = null;
        foreach ($messages as $message) {
            $fixed = ($message['fixed']) ? 'message-fixed' : null;
            $html .= '
                <div class="messages message-'.$type.' '.$fixed.'">'.$message['message'];
            $html .= '</div>';
        }
        
        return $html;
    }
    
    /**
     * A simple recursive array_search function
     * @param type $needle
     * @param type $haystack
     * @return boolean
     */
    function recursive_array_search($needle,$haystack) {
        foreach($haystack as $key=>$value) {
            $current_key=$key;
            if($needle===$value OR (is_array($value) && $this->recursive_array_search($needle,$value) !== false)) {
                return $current_key;
            }
        }
        return false;
    }
}

/* End of file Messages.php */
/* Location: ./application/libraries/Messages.php */