<?php 
/**
 * Model taula 'notaseguiment'
 * Basat en MY_Model (funcionalitat CRUD)
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Notaseguiment_model extends MY_Model
{
    public $_table = 'notaseguiment';
    public $primary_key = 'id_notaSeguiment';
    public $before_create = array( '_create_timestamp' );
    
    public function __construct()
    {
        parent::__construct();

    }
    
    /**
     * Recupera dades notes de seguiment segons tipus i incidència
     * @param {string} $id_incidencia
     * @param {string} $tipus
     * @return {object}
     */
    public function get_notes_seguiment($id_incidencia, $tipus = '')
    {
        if (!$tipus)
        {
            if ($this->session->userdata('usuari')->rol === 'solicitant')
                $tipus = array('solicitant', 'feedback');
            else
                $tipus = array('solicitant', 'feedback', 'gestor');
        }
        
        $this->db->select('notaseguiment.*, concat(usuari.nom," ",usuari.cognoms) usuari_nom, (CASE WHEN usuari.foto IS NULL THEN "generica.png" ELSE usuari.foto END) as usuari_foto', FALSE);
        $this->db->from('notaseguiment');
        $this->db->join('usuari', 'usuari.id_usuari = notaseguiment.added_by');
        $this->db->where('incidencia', $id_incidencia);
        $this->db->where_in('tipus', $tipus);
        $this->db->order_by('added_on', 'desc');
        return $this->db->get()->result();
    }
    
    protected function _create_timestamp($data)
    {
       $data['added_on'] = date('Y-m-d H:i:s');
       $data['added_by'] = $this->session->userdata('usuari')->id_usuari;
       return $data;
    }

}