<?php 
/**
 * Model taula 'preguntafrequent' i relacionades
 * Basat en MY_Model (funcionalitat CRUD)
 */

if (!defined('BASEPATH')) exit('Accés denegat');

class Pf_model extends MY_Model
{

    public $_table = 'preguntafrequent';
    public $primary_key = 'id_preguntaFrequent';
    
    public function __construct()
    {
        parent::__construct();

    }
    
    /**
     * Obté pregunta frequent passada per paràmetre
     * @param {string} $id Id PF
     * @return {object} PF
     */
    public function get_pf($id)
    {
        $this->db->select('preguntafrequent.*');
        $this->db->from('preguntafrequent');
        $this->db->where(array('preguntafrequent.id_preguntafrequent' => $id));
        
        $res = $this->db->get();
        return $res->row();
    }
    
    /**
     * Obté motius relacionats amb una PF
     * @param {string} $id_pf Id PF
     * @return {object} Motius
     */
    public function get_motius_relacionats($id_pf)
    {
        $this->db->select('motiu.id_motiu id_motiu, motiu.descripcio motiu');
        $this->db->from('motiu');
        $this->db->join('preguntafrequent_motius', 'preguntafrequent_motius.id_motiu = motiu.id_motiu');
        $this->db->where(array('id_preguntafrequent' => $id_pf));
        $q = $this->db->get();
        
        $motius_relacionats = array();
        foreach($q->result() as $motiu) { $motius_relacionats[] = $motiu->id_motiu; }
        return $motius_relacionats;
    }
    
    /**
     * Obté PFs relacionades amb un motiu
     * @param {string} $id_motiu Id motiu
     * @return {object} PFs
     */
    public function get_pfs_relacionades($id_motiu)
    {
        $this->db->select('preguntafrequent.*');
        $this->db->from('preguntafrequent');
        $this->db->join('preguntafrequent_motius', 'preguntafrequent_motius.id_preguntaFrequent = preguntafrequent.id_preguntaFrequent');
        $this->db->where(array('id_motiu' => $id_motiu));
        $this->db->order_by('visites','desc');
        return $this->db->get()->result();
    }
    
    /**
     * Obté preguntes freqüents segons filtres donats
     * @param {array} $filtres Array de filtres
     * @param {string} $limit Màxim de pfs a retornar
     * @param {string} $inici Punter inici pel qual començar a retornar pfs
     * @return {array} PFs
     */
    public function get_pfs($filtres = array(), $limit = '0', $inici = '0', $order_by = 'added_on')
    {
        $this->db->select('preguntafrequent.*');
        $this->db->from('preguntafrequent');
        if (@$filtres['text'])
        {
            $text = $this->db->escape_like_str($filtres['text']);
            $this->db->where('(pregunta LIKE "%'.$text.'%" OR solucio LIKE "%'.$text.'%" OR etiquetes LIKE "%'.$text.'%")', NULL, TRUE);
        }

        $this->db->order_by($order_by, 'desc');
        
        if ($limit!='0' || $inici!='0')
            $this->db->limit($limit, $inici);
        
        $res = $this->db->get();
        return $res->result();
    }
    
    /**
     * Guarda pregunta freqüent amb les dades enviades per formulari
     * @param {Array} $post Dades enviades per formulari
     * @return {String} ID nova pf
     */
    public function insert_pf($input)
    {
        $data = array(
            'pregunta' => nl2br($input['pregunta']),
            'solucio' => nl2br($input['solucio']),
            'etiquetes' => $input['etiquetes']
        );
        $this->db->insert('preguntafrequent', $this->_create_timestamp($data));
        $id = $this->db->insert_id();
        
        foreach ($input['motiu'] as $motiu)
        {
            $this->db->insert('preguntafrequent_motius', array('id_preguntafrequent'=>$id, 'id_motiu'=>$motiu));
        }
        return $id;
    }
    
    /**
     * Actualitza PF amb les dades enviades per formulari
     * @param {Array} $post Dades enviades per formulari
     * @return {boolean}
     */
    public function upd_pf($input)
    {
        $data = array(
            'pregunta' => $input['pregunta'],
            'solucio' => $input['solucio'],
            'etiquetes' => $input['etiquetes']
        );
        $this->db->where('id_preguntaFrequent', $input['id_preguntaFrequent']);
        $this->db->update('preguntafrequent', $this->_update_timestamp($data));
        
        $this->db->delete('preguntafrequent_motius', array('id_preguntaFrequent'=>$input['id_preguntaFrequent']));
        foreach ($input['motiu'] as $motiu)
        {
            $this->db->insert('preguntafrequent_motius', array('id_preguntaFrequent'=>$input['id_preguntaFrequent'], 'id_motiu'=>$motiu));
        }
        
        return true;
    }
    
    /**
     * Esborra PF de la base de dades
     * @param {string} $id_preguntaFrequent Id PF
     * @return {boolean}
     */
    public function delete_pf($id_preguntaFrequent)
    {
        $this->db->delete('preguntafrequent_motius', array('id_preguntaFrequent'=>$id_preguntaFrequent));
        $this->db->delete('preguntafrequent', array('id_preguntaFrequent'=>$id_preguntaFrequent));
        return true;
    }
    
    /**
     * Augmenta el num de visites en 1 d'una PF
     */
    public function registra_visita($id_preguntaFrequent)
    {
        $this->db->where('id_preguntaFrequent', $id_preguntaFrequent);
        $this->db->set('visites', 'visites+1', FALSE);
        $this->db->update('preguntafrequent');
    }

    protected function _create_timestamp($data) {
       $data['added_on'] = $data['updated_on'] = date('Y-m-d H:i:s');
       $data['updated_by'] = $this->session->userdata('usuari')->id_usuari;
       return $data;
    }

    protected function _update_timestamp($data) {
       $data['updated_on'] = date('Y-m-d H:i:s');
       $data['updated_by'] = $this->session->userdata('usuari')->id_usuari;
       return $data;
    }
}

/* Fi del fitxer pf_model.php */
/* Ubicació: ./application/models/pf_model.php */