SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';

CREATE SCHEMA IF NOT EXISTS `ciau` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci ;
USE `ciau` ;

-- -----------------------------------------------------
-- Table `ciau`.`departament`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`departament` (
  `id_departament` INT NOT NULL AUTO_INCREMENT ,
  `nom` VARCHAR(100) NULL ,
  PRIMARY KEY (`id_departament`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`rol`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`rol` (
  `id_rol` INT NULL AUTO_INCREMENT ,
  `nom` VARCHAR(45) NULL ,
  `descripcio` VARCHAR(200) NULL ,
  PRIMARY KEY (`id_rol`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`usuari`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`usuari` (
  `id_usuari` VARCHAR(15) NOT NULL ,
  `contrasenya` VARCHAR(100) NULL ,
  `rol` INT NOT NULL ,
  `nom` VARCHAR(45) NOT NULL ,
  `cognoms` VARCHAR(100) NULL ,
  `email` VARCHAR(100) NOT NULL ,
  `foto` VARCHAR(15) NULL ,
  `departament` INT NULL ,
  `data_baixa` DATE NULL ,
  `added_on` DATETIME NULL ,
  `updated_on` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP ,
  `updated_by` VARCHAR(15) NULL ,
  PRIMARY KEY (`id_usuari`) ,
  INDEX `fk_usuari_departament` (`departament` ASC) ,
  INDEX `fk_usuari_rol_idx` (`rol` ASC) ,
  CONSTRAINT `fk_usuari_departament`
    FOREIGN KEY (`departament` )
    REFERENCES `ciau`.`departament` (`id_departament` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_usuari_rol`
    FOREIGN KEY (`rol` )
    REFERENCES `ciau`.`rol` (`id_rol` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`motiu`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`motiu` (
  `id_motiu` INT NULL AUTO_INCREMENT ,
  `descripcio` VARCHAR(100) NULL ,
  `id_departament` INT NOT NULL ,
  PRIMARY KEY (`id_motiu`) ,
  INDEX `fk_motiu_departament1_idx` (`id_departament` ASC) ,
  CONSTRAINT `fk_motiu_departament1`
    FOREIGN KEY (`id_departament` )
    REFERENCES `ciau`.`departament` (`id_departament` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`estat`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`estat` (
  `id_estat` INT NOT NULL ,
  `nom` VARCHAR(20) NULL ,
  `descripcio` VARCHAR(200) NULL ,
  PRIMARY KEY (`id_estat`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`incidencia`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`incidencia` (
  `id_incidencia` INT NOT NULL AUTO_INCREMENT ,
  `solicitant` VARCHAR(15) NOT NULL ,
  `motiu` INT NOT NULL ,
  `titol` VARCHAR(255) NOT NULL ,
  `descripcio` TEXT NOT NULL ,
  `prioritat` SET('baixa','normal','alta') NOT NULL DEFAULT 'normal' ,
  `fitxer_adjunt` VARCHAR(200) NULL COMMENT 'Nom del fitxer adjunt' ,
  `estat` INT NOT NULL ,
  `departament_assignat` INT NOT NULL COMMENT 'Departament assignat' ,
  `gestor_assignat` VARCHAR(15) NULL ,
  `added_on` DATETIME NOT NULL ,
  `updated_on` TIMESTAMP NULL ,
  `updated_by` VARCHAR(15) NULL ,
  PRIMARY KEY (`id_incidencia`) ,
  INDEX `fk_usuari_incidencia_idx` (`solicitant` ASC) ,
  INDEX `fk_incidencia_usuari1_idx` (`gestor_assignat` ASC) ,
  INDEX `fk_incidencia_motiu1_idx` (`motiu` ASC) ,
  INDEX `fk_incidencia_estat1_idx` (`estat` ASC) ,
  INDEX `fk_incidencia_departament1_idx` (`departament_assignat` ASC) ,
  CONSTRAINT `fk_incidencia_usuari_solicitant`
    FOREIGN KEY (`solicitant` )
    REFERENCES `ciau`.`usuari` (`id_usuari` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_incidencia_usuari_gestor`
    FOREIGN KEY (`gestor_assignat` )
    REFERENCES `ciau`.`usuari` (`id_usuari` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_incidencia_motiu1`
    FOREIGN KEY (`motiu` )
    REFERENCES `ciau`.`motiu` (`id_motiu` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_incidencia_estat1`
    FOREIGN KEY (`estat` )
    REFERENCES `ciau`.`estat` (`id_estat` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_incidencia_departament1`
    FOREIGN KEY (`departament_assignat` )
    REFERENCES `ciau`.`departament` (`id_departament` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`actualitzacio`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`actualitzacio` (
  `id_actualitzacio` INT NOT NULL AUTO_INCREMENT ,
  `incidencia` INT NOT NULL ,
  `tipus` SET('estat','assignacio','prioritat') NOT NULL ,
  `abans` VARCHAR(100) NULL ,
  `despres` VARCHAR(100) NULL ,
  `added_on` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
  `added_by` VARCHAR(15) NOT NULL ,
  PRIMARY KEY (`id_actualitzacio`) ,
  INDEX `fk_actualitzacio_incidencia1_idx` (`incidencia` ASC) ,
  INDEX `fk_actualitzacio_usuari1_idx` (`added_by` ASC) ,
  CONSTRAINT `fk_actualitzacio_incidencia1`
    FOREIGN KEY (`incidencia` )
    REFERENCES `ciau`.`incidencia` (`id_incidencia` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_actualitzacio_usuari1`
    FOREIGN KEY (`added_by` )
    REFERENCES `ciau`.`usuari` (`id_usuari` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`preguntaFrequent`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`preguntaFrequent` (
  `id_preguntaFrequent` INT NOT NULL AUTO_INCREMENT ,
  `pregunta` TEXT NOT NULL ,
  `solucio` TEXT NOT NULL ,
  `etiquetes` VARCHAR(200) NULL COMMENT 'Paraules clau separades per coma' ,
  `visites` INT NULL DEFAULT 0 ,
  `added_on` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP ,
  `updated_by` VARCHAR(15) NOT NULL ,
  `updated_on` DATETIME NULL ,
  PRIMARY KEY (`id_preguntaFrequent`) ,
  INDEX `fk_preguntaFrequent_usuari1_idx` (`updated_by` ASC) ,
  CONSTRAINT `fk_preguntaFrequent_usuari1`
    FOREIGN KEY (`updated_by` )
    REFERENCES `ciau`.`usuari` (`id_usuari` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`preguntaFrequent_motius`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`preguntaFrequent_motius` (
  `id_preguntaFrequent` INT NOT NULL ,
  `id_motiu` INT NOT NULL ,
  PRIMARY KEY (`id_preguntaFrequent`, `id_motiu`) ,
  INDEX `fk_preguntaFrequent_has_motiu_motiu1_idx` (`id_motiu` ASC) ,
  INDEX `fk_preguntaFrequent_has_motiu_preguntaFrequent1_idx` (`id_preguntaFrequent` ASC) ,
  CONSTRAINT `fk_preguntaFrequent_has_motiu_preguntaFrequent1`
    FOREIGN KEY (`id_preguntaFrequent` )
    REFERENCES `ciau`.`preguntaFrequent` (`id_preguntaFrequent` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_preguntaFrequent_has_motiu_motiu1`
    FOREIGN KEY (`id_motiu` )
    REFERENCES `ciau`.`motiu` (`id_motiu` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`notaSeguiment`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`notaSeguiment` (
  `id_notaSeguiment` INT NOT NULL AUTO_INCREMENT ,
  `incidencia` INT NOT NULL ,
  `tipus` SET('solicitant','gestor','feedback','solucio') NOT NULL ,
  `descripcio` TEXT NOT NULL ,
  `valoracio` TINYINT NULL COMMENT 'Valoració d\'una solució. 1 :-) - 2 :-(' ,
  `added_on` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
  `added_by` VARCHAR(15) NOT NULL ,
  PRIMARY KEY (`id_notaSeguiment`) ,
  INDEX `fk_notaSeguiment_incidencia1_idx` (`incidencia` ASC) ,
  INDEX `fk_notaSeguiment_usuari1_idx` (`added_by` ASC) ,
  CONSTRAINT `fk_notaSeguiment_incidencia1`
    FOREIGN KEY (`incidencia` )
    REFERENCES `ciau`.`incidencia` (`id_incidencia` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE,
  CONSTRAINT `fk_notaSeguiment_usuari1`
    FOREIGN KEY (`added_by` )
    REFERENCES `ciau`.`usuari` (`id_usuari` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `ciau`.`verificaEmail`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `ciau`.`verificaEmail` (
  `usuari` VARCHAR(15) NOT NULL ,
  `nou_email` VARCHAR(100) NOT NULL ,
  `codi_verificacio` VARCHAR(45) NOT NULL ,
  `added_on` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP ,
  INDEX `fk_verificaMail_usuari1_idx` (`usuari` ASC) ,
  PRIMARY KEY (`usuari`) ,
  CONSTRAINT `fk_verificaMail_usuari1`
    FOREIGN KEY (`usuari` )
    REFERENCES `ciau`.`usuari` (`id_usuari` )
    ON DELETE NO ACTION
    ON UPDATE CASCADE)
ENGINE = InnoDB;

USE `ciau` ;


SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;
